	LIST p=PIC12F629,n=80,t=ON,st=OFF,x=OFF
	ERRORLEVEL 1
	INCLUDE <C:\PIC\P12F629.INC>
;
; New Single-Wire Cave Telephone Controller.  May 2009.  V7 - Feb 2013.
; Author - Stuart France
;
; **** COMMENT OUT LINE BELOW OUT TO COMPILE THE BASE STATION VERSION ****
#define HANDSET 1
; HANDSET must be defined to compile the cave handset version (4.5V)
; HANDSET is undefined to compile the base station version (4.5 or 12V)
; **********************************************************************
;
	NOLIST
;
; Uses internal 4MHz clock on P12 device.  Watchdog timer not used.
; Runs on 3xAAs for handsets, 3xD cells for a basestation box using the
; MAX756 DC-DC converter, or use an external 12v lead acid battery for
; basestation boards that have a 78L05 to supply the logic side.
; Timer interrupts occur at 16Hz.
; Push button is on the uP INT pin which causes an interrupt too.
; Brown-out detect (BOD) is enabled which uses 90uA even in SLEEP, however
; this ensures that a uP reset occurs when new batteries are inserted.
; With brown-out disabled it would uses <1uA in sleep but a software crash
; might then occur due to fumbling while inserting the batteries.
;
; User interface for cave handsets. How to use it:
; 1) Fit the batteries, you hear a roger, goes into sleep mode after
;    10 seconds if you fail to give the 'power on' command promptly.
; 2) Hold button down firmly for 3 seconds to 'power on', you hear
;    a start-tune as it wakes from sleep, then it stays powered on.
;    Button held down throughout start-tune sets Basestation mode.
; 3) Hold button to talk, led on while talking, release to listen,
;    roger tone when released.  Led flashes every 10 secs.  If the
;    batteries are low you see a double led flash instead of a single.
; 4) Double-click button to send call-tones.  
;    Click button to end tones, roger bleep confirms.  If running in
;    basestation mode, call-tones will change to pips after 10secs.
; 5) Hold button down firmly for 3 seconds while in call-tones mode
;    to turn it if off, i.e. go to sleep. You hear the shutdown sound.
;    Goes into sleep after few hours unless running as a base station.
;
; User interface for surface base station box:
; 1) Use the switch on volume knob to turn it on or off
; 2) Hold down the button to talk, release to listen
; 3) Double click to start calltones which change to pips after 10sec.
;    Click again to stop the call tones or pips.
;
; ***************************** GPIO PORT **************************
;
	LIST
;
#define TRISX   b'00001100'    ; i/o port settings
#define OPTX    b'11000111'    ; option mask 12F, pull ups off
; assign prescaler to TMR0 at x256 scaling, INT on rising edge
#define CONX    b'00000000'    ; INTCON mask 12F, INT pin disabled
#define IOCBX   b'00000000'    ; no interrupts on any pin change
#define CMCONX  b'00000111'    ; comparator off config
#define VRCONX  b'00000000'	   ; Vref generator - off
#define PIE1X   b'00000000'	   ; Peripheral interrupts - none
#define T1CONX	b'00000000'	   ; Timer1 module - off
#define INTSEC  0x10		   ; 16 interrupts/sec;
#define PSUPIN  GPIO,0         ; 12V power supply enable (high=on)
#define LEDPIN  GPIO,1         ; LED indicator pin (high=on)
#define BUTPIN  GPIO,2         ; Push button (high=pressed)
#define BATPIN  GPIO,3         ; Battery level pin (high=good)
#define RLAPIN  GPIO,4         ; TX relay on pin (high=driven)
#define BUZPIN  GPIO,5         ; Tone generator pin (high=driven)
;
; ***************************** USER RAM *****************************
;
RAMLO	EQU		0x20	   ; lowest location in user ram           
DELCNT  EQU     0x20       ; delay counter
WTEMP   EQU     0x21       ; w save in interrupts
STEMP   EQU     0x22       ; status save in interrupts
SECTMR  EQU     0x23       ; seconds countdown
S16TMR  EQU     0x24       ; 16ths of sec countdown
FLAGS   EQU     0x25       ; flag word comprising:
TMRFLG  EQU     0          ;   timer interrupt has occurred
BUTFLG  EQU		1		   ;   button press interrup occurred
BGAPFL	EQU		2		   ;   in a gap between button presses
LEDFLG	EQU		3		   ;   show a led pip
TIMOUT	EQU		4		   ;   button down timeout
BSTNFL	EQU		5		   ;   base station mode (for handsets)
ASDFLG	EQU		6		   ;   auto shutdown flag
OFFDEL	EQU		0x26	   ; for making a dying shutdown tone
PSTATE	EQU		0x27	   ; phone state number
BTDNTM	EQU		0x28	   ; Button down time (in 16ths sec)
BTUPTM	EQU		0x29	   ; Button up time (ditto)
LEDTMR	EQU		0x2A	   ; Time left before next LED pip (ditto)
PIPGAP	EQU		0x2B	   ; Qtr secs counter between pips
TEMP1	EQU		0x2C	   ; General temp byte
TONCNT	EQU		0x2D	   ; Times to send call tone before timeout
UPTIMR	EQU		0x2E	   ; Count up timer (int)
MAXDWN	EQU		0x2F	   ; Max button down time (int)
TONTMP	EQU		0x30	   ; Tone generation temp
SHUTHI	EQU		0x31	   ; Auto shutdown counter (hi, medium, low)
SHUTME	EQU		0x32	   ;    this is a 32 bit up-counter
SHUTLO	EQU		0x33
RAMHI	EQU		0x5F	   ; highest location in user RAM
;
; ***************************** LITERALS *****************************
;
S_SHDN	EQU		0			; shutdown state
S_WAIT	EQU		1			; waiting for 3s press to 'turn on'
S_ONRX	EQU		2			; turned on and receiving
S_ONTX	EQU		3			; turned on and transmitting
S_TONE	EQU		4			; sending call tone
L_TIME	EQU		d'160'		; LED pip interval in 16ths during rx state
P_TIME	EQU		d'20'		; Tone pip interval in qtr sec
S_TIME  EQU		d'160'		; Period looking for long press in 16ths
O_TIME	EQU		d'48'		; Length of a 'long' press in 16ths
NTONES	EQU		d'10'		; number of call tones after a double-click
;
; ************************** MACROS ***********************************
;
BFNE    macro   fil,lit,lab     ; If file <> literal then goto label
        movf    fil,W
        xorlw   lit
        btfss   STATUS, Z
        goto    lab
        endm
;
BFZE    macro   fil,lab         ; If file = 0 then goto lab
        movf    fil, W
        btfsc   STATUS, Z
        goto    lab
        endm
;
BFNZ    macro   fil,lab         ; If file <> 0 then goto lab
        movf    fil, W
        btfss   STATUS, Z
        goto    lab
        endm
;
BFGEL   macro fil1,lit,lab     ; if file1 >= lit then goto lab
        movlw   lit
        subwf   fil1,W         ; w = fil1 - lit
        btfsc   STATUS,C       ;
        goto    lab
        endm
;
; ************************* RESET ***********************************
;
	ORG     0              ; reset vector
    goto    START
;
; *************************** INTERRUPT *****************************
;
    ORG     4              ; interrupt servicing (16Hz, 62ms clock)
	movwf	WTEMP          ; save working registers
	swapf	STATUS,W
	bcf     STATUS,RP0
	movwf   STEMP
	movf    GPIO,W         ; re-read GPIO to clear changes
;
INTTMR
    btfss   INTCON,T0IF    ; it is a timer interrupt?
    goto    INTINT         ;    no, check for button press (INT pin)
;
; handle a timer interrupt
	bsf		FLAGS,TMRFLG   ; flag that a timer interrupt has occurred
	bcf		INTCON,T0IF	   ; clear timer int flag
;
; increment UPTIMR to a maximum value of 80 (5 secs)
	BFGEL	UPTIMR,d'80',NOTIMR
	incf	UPTIMR,F
NOTIMR
	bcf		FLAGS,LEDFLG	; led should go off by default
	btfsc	BATPIN			; battery low causes a double LED flash
	goto	BTGOOD			;   - was not low
	BFNE	LEDTMR,d'4',BTGOOD
	bsf		FLAGS,LEDFLG   ; do a led pip
BTGOOD
	decfsz	LEDTMR,F
	goto	NOTDUE
	movlw	L_TIME
	movwf	LEDTMR
	bsf		FLAGS,LEDFLG   ; do a led pip
NOTDUE
	BFNE	PSTATE,S_ONRX,NOPIP
	btfss	FLAGS,LEDFLG	; led pip needed (done in rx only)?
	bcf		LEDPIN			;    no, led off
	btfsc	FLAGS,LEDFLG	; led pip not needed?
	bsf		LEDPIN			;    no, led on
NOPIP
	BFZE    S16TMR,NODEC   ; dec S16TMR timer if nonzero
    decf    S16TMR,F
;
NODEC
	btfss	BUTPIN
	goto	INCSHT
	clrf	SHUTHI			; zero the auto-shutdown counter
	clrf	SHUTME			; which is 24 bits wide - HI/ME/LO
	clrf	SHUTLO          ; if button is pressed
INCSHT
	incf	SHUTLO,F		; inc auto shutdown counter
	btfss	STATUS,Z		; rollover?
	goto	INTEND			; no
	incf	SHUTME,F
	btfss	STATUS,Z
	goto	INTEND
	incf	SHUTHI,F
	btfsc	SHUTHI,3		; i.e. =8, this gives about 9 hours
	bsf		FLAGS,ASDFLG	; then signal an auto shutdown
	goto	INTEND
;
INTINT
    btfss   INTCON,INTF    ; button press (INT pin) interrupt?
    goto    INTEND         ;    no, other types should not happen!
	clrf	BTDNTM		   ; clear button down-up timers
	clrf	BTUPTM
	bcf		INTCON,INTF	   ; clear INT flag
; fall through to INTEND ...
;
INTEND
;	btfsc	SHUTME,1		; DEBUG: this is equiv to 32 secs
;	bsf		FLAGS,ASDFLG	; signal an auto shutdown
;
    bcf     INTCON,GPIF    ; clear GPIO-changed flag anyway
	swapf   STEMP,W        ; restore working registers
	movwf   STATUS
	swapf	WTEMP,F
	swapf	WTEMP,W
    retfie
;
; ************************ MAIN PROGRAM *****************************
;
START
    call    INIVAR
	clrf	GPIO
	call    INITIO
	clrf	GPIO
	call	INTRAN			; settle down
	call	INTRAN
;
#ifdef HANDSET
	call	PSUON			; 12v on, test LED and spkr
	call	ROGRTN			; pip
	call	PSUOFF			; turn off until set is needed
#else
	call	SECDEL			; let voltage stabilise
	bsf		LEDPIN			; LED on (it will go off at next INT)
	call	PSUON			; turn 12v on and this stays on
	call	SECDEL			; let voltage stabilise
	call	STRTTN			; Play Beethoven
	bcf		LEDPIN			; LED off
	bsf		FLAGS,BSTNFL	; run in base station mode
#endif
;
; ---------------- MAIN EVENT LOOP --------------
;
EVLOOP
#ifdef HANDSET
	BFNE	PSTATE,S_SHDN,EVWAIT	; no, goto EVWAIT if 'on'
;
SHUTDN
	clrf	GPIO				; turn peripherals off
	movlw	S_SHDN				; set off-state
	movwf	PSTATE
	bsf		INTCON,INTE			; enable an INT interrupt
	nop
	SLEEP
	nop							; button was pressed (INT)
	bcf		INTCON,INTE			; no more INT interupts
	nop
	movlw	S_WAIT				; set wait-state
	movwf	PSTATE				; waiting for a 3-second press
	movlw	S_TIME				; time for the long button press
	movwf	S16TMR				; Long button press is needed
	goto	EVLOOP				; in order to move to the onrx-state
;
EVWAIT
	; code to turn on the phone after a 3sec button press
	BFNE	PSTATE,S_WAIT,EVTXRX ; goto EVTXRX if not waiting
	BFZE	S16TMR,SHUTDN		; on button timeout shutdown
	movlw	O_TIME				; set minimum press time
	call	BUTDWN				; get length of a button press
	BFZE	BTDNTM,EVLOOP		; button not pressed at all
	btfss	FLAGS,TIMOUT		; a timeout should occur
	goto	EVLOOP				; it didn't
EVWT11
	bsf		FLAGS,BSTNFL		; assume base-station mode wanted
	bcf		LEDPIN				; LED off
	call	PSUON				; 12v on
	call	QSECDL
	call	STRTTN				; play Beethoven - STRTTN will
; clear BSTNFL if button is released before the end of the tune
; in which case the handset was started in cave-handset mode
EVWT22
	call	BUTUP				; wait for button release
	clrf	S16TMR				; clear any pending timer
	movlw	L_TIME				; 
	movwf	LEDTMR				; reload led time for next pip
	movlw	S_ONRX				; set on-receive state
	movwf	PSTATE
	goto	EVLOOP
#endif
;
EVTXRX
	; phone is now 'on' and in a receive or a transmit state
	BFNE	PSTATE,S_ONTX,EVRX
EVTX
	; transmitting speech, will only return to rx from here
	bsf		LEDPIN				; led on while transmitting speech
	bsf		RLAPIN				; PTT relay on
	clrf	UPTIMR				; start a new count-up (int) timer
EVTX1
	btfsc	BUTPIN				; button up?
	goto	EVTX1				;   no, carry on transmitting
TXBTUP
	; button was released during tx.  Button ought now to be up.
	BFGEL	UPTIMR,d'8',NOTDCL	; >0.5s gap means not a double-click
	call	INTRAN				; debounce button-up, 1-62msec variable
    call	INTRAN				; then a 62ms fixed time
;
	movlw	d'8'
	movwf	S16TMR				; start new 0.5s count-down timer
TXBT01
	BFZE	S16TMR,NOTDCL		; out of time for 2nd click -> exit tx
	btfss	BUTPIN				; button down (for the 2nd click) ?
	goto	TXBT01				;   no, carry on looking for it
	call	BUTUP				; wait for button release
	call	QSECDL				; debounce
	bcf		LEDPIN				; led off
	bcf		RLAPIN				; relay off
	goto	EVTONE				; start call tones
;
NOTDCL
	movlw	L_TIME 
	movwf	LEDTMR				; reload led time for next pip
	bcf		LEDPIN				; led off
	bcf		RLAPIN				; relay off
	call	QSECDL				; allow time for relay to drop
	call	ROGRTN				; Roger bleep
	movlw	S_ONRX				; return to rx state
	movwf	PSTATE
	goto	EVLOOP
;
EVRX
	; receiving speech, can go to rx from here
	BFNE	PSTATE,S_ONRX,EVTONE
EVRX0
#ifdef HANDSET
	btfsc	FLAGS,BSTNFL		; base station mode?
	goto	EVRX1				;   yes, so never do an auto-shutdown
	btfsc	FLAGS,ASDFLG		; auto shutdown required?
	goto	EVCLOS				;   yes, play raspberry etc
#endif
EVRX1
	btfss	BUTPIN				; button down?
	goto	EVRX0				;   no, tight loop, rx & led pips
EVRX2
	movlw	S_ONTX				; enter the tx state
	movwf	PSTATE
	goto	EVLOOP
;
EVTONE
	; call tone state, can return to rx or turn off from here
	movlw	S_ONRX				; tones done from rx state
	movwf	PSTATE
	movlw	NTONES
	movwf	TONCNT				; send N proper call tones, then pips
	incf	TONCNT,F
EVTN1
	decfsz	TONCNT,F
	goto	EVTN2
;
	btfsc	FLAGS,BSTNFL
	goto	EVPIPS				; goto evpips if calltones lead to pips
	goto	ENDTON				; goto endton if not a base station
;
EVTN2
	call	CALLTN				; send a tone pattern, abort on button press
	movlw	O_TIME				; set timeout for a long button press
	call	BUTDWN				; get length of a button press
#ifdef HANDSET
	btfsc	FLAGS,TIMOUT		; button held down for over 3s?
	goto	EVCLOS				;    yes, so shutdown
#endif
	BFNZ	BTDNTM,EVPIP3		; button held for a bit less?
	goto	EVTN1				;    no button press at all, do another tone
;
EVPIPS
	movlw	P_TIME				; gap in qs between pips
	movwf	PIPGAP
EVPIP1
	decfsz	PIPGAP,F			; time for another pip?
	goto	EVPIP2				;    no
	call	ROGRTN				;    yes, send a pip
	movlw	P_TIME				; set new gap between pips
	movwf	PIPGAP
EVPIP2
	call	QSECDB				; wait qtr sec unless button down
	movlw	O_TIME				; set timeout press time in 16ths
	call	BUTDWN				; abort on any kind of button press
#ifdef HANDSET
	btfsc	FLAGS,TIMOUT		; button held down causing 3s timeout?
	goto	EVCLOS				;    yes, do shutdown
#endif
	BFNZ	BTDNTM,EVPIP3		; button held a bit?
	goto	EVPIP1				;    no, do another tone
EVPIP3
	call	ROGRTN
ENDTON
	call	BUTUP				; but sure button is up
	goto	EVLOOP
;
#ifdef HANDSET
EVCLOS
	bcf		INTCON,GIE			; no interrupts
	nop
	clrf	SHUTHI				; zero the auto-shutdown counter
	clrf	SHUTME				; 
	clrf	SHUTLO
	bcf		FLAGS,ASDFLG		; no auto shutdown
	bsf		INTCON,GIE			; allow interrupts
	nop
	call	SHUTTN				; logical shutdown
	movlw	S_SHDN
	movwf	PSTATE
	goto	EVLOOP
#endif
;
;
;
; *************************** PROCEDURES ****************************
;
;
BUTUP
	call	INTRAN
	btfsc	BUTPIN
	goto	BUTUP
	return
;
;
BUTDWN							; measures button down time (if any)
	movwf	MAXDWN				; save max down time wanted
	bcf		FLAGS,TIMOUT
	btfss	BUTPIN				; button down?
	goto	NOTDWN				;   no
	clrf	UPTIMR				;   yes, start the count
BUTDN1
	btfss	BUTPIN				; button still down?
	goto	BUTDN3				;   no, exit with the count
	movf	UPTIMR,W			;   yes...
    xorwf   MAXDWN,W			; count = max ?
    btfss   STATUS,Z
    goto    BUTDN1				;   no, wait for release
	bsf		FLAGS,TIMOUT		;   yes, exit with uptimr at max
BUTDN3
	movf	UPTIMR,W			; button might still be down here
	movwf	BTDNTM				; save button down time
	return
NOTDWN
	clrf	BTDNTM				; button never pressed at all
	return
;
;
CALLTN							; send one call tone pattern
	movlw	.2
	movwf	S16TMR
BUZ001
	bsf		BUZPIN
	movlw	.50
	call	GAP2
	bcf		BUZPIN
	movlw	.50
	call	GAP2
	btfsc	FLAGS,BUTFLG		; abort if button pressed
	return
	BFNZ	S16TMR,BUZ001
;
	movlw	.2
	movwf	S16TMR
BUZ002
	bsf		BUZPIN
	movlw	.60
	call	GAP2
	bcf		BUZPIN
	movlw	.60
	call	GAP2
	btfsc	FLAGS,BUTFLG		; abort if button pressed
	return
	BFNZ	S16TMR,BUZ002
;
	movlw	.4
	movwf	S16TMR
;
BUZ003
	bsf		BUZPIN
	movlw	.50
	call	GAP2
	bcf		BUZPIN
	movlw	.50
	call	GAP2
	btfsc	FLAGS,BUTFLG		; abort if button pressed
	return
	BFNZ	S16TMR,BUZ003
;
	movlw	.20
	movwf	S16TMR
BUZ005
	btfsc	BUTPIN				; abort if button pressed
	return
	BFNZ	S16TMR,BUZ005
	return
;
; ----------------
;
SHUTTN							; shutdown tone pattern
	movlw	.32
	movwf	S16TMR
	movlw	.20
	movwf	OFFDEL
SHT001
	bsf		BUZPIN
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	bcf		BUZPIN
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	movf	OFFDEL,W
	call	GAP
	incf	OFFDEL,F
	BFNZ	S16TMR,SHT001
	return
;
; -----------------------
;
STRTTN							; Start-up tone pattern
	movlw	.20					; Beethoven's 9th (ode to joy)
	call	ROGER2				; button held down throughout this
	movlw	.20					; tune means 'be a base station'
	call	ROGER2
	movlw	.15				
	call	ROGER2
	movlw	.8				
	call	ROGER2
	movlw	.8				
	call	ROGER2
	movlw	.15					
	call	ROGER2
	movlw	.20				
	call	ROGER2
	movlw	.30				
	call	ROGER2
	movlw	.40				
	call	ROGER2
	movlw	.40				
	call	ROGER2
	movlw	.30				
	call	ROGER2
	movlw	.20				
	call	ROGER2
	movlw	.20				
	call	ROGER2
	movlw	.30				
	call	ROGER2
	movlw	.30				
	call	ROGER2
	return
;
; -----------------------
;
ROGER2
	btfss	BUTPIN				; if button is not kept down
	bcf		FLAGS,BSTNFL		; to end of Beethoven tune
	goto	ROGER1				; then don't be a base station
;
ROGRTN							; roger tone
	movlw	.20
	goto	ROGER1
;
ROGER1							; general entry point to do a
	movwf	TONTMP				; roger beep at a given pitch
	movlw	.3
	movwf	S16TMR
ROG001
	bsf		BUZPIN
	movf	TONTMP,W
	call	GAP
	bcf		BUZPIN
	movf	TONTMP,W
	call	GAP
	BFNZ	S16TMR,ROG001
	movlw	.50
	call	GAP
	return
;
; ----------------
;
PSUON							; turn 12v psu on
	bsf		PSUPIN
	movlw	.4
	movwf	S16TMR
PSUON1
	BFNZ	S16TMR,PSUON1
	return
;
; ----------------
;
PSUOFF							; turn 12v psu off
	bcf		PSUPIN
	movlw	.4
	movwf	S16TMR
PSUOF1
	BFNZ	S16TMR,PSUOF1
	return
;
; -----------------
;
GAP2
	bcf		FLAGS,BUTFLG
	movwf	DELCNT
GAP2A
	goto    $+1
	goto    $+1
	btfsc	BUTPIN				; button down?
	bsf		FLAGS,BUTFLG		; 	yes, abort if button down
	btfsc	FLAGS,BUTFLG		;        and flag set
	return
	decfsz	DELCNT,F
	goto    GAP2A
	return						; return with flag unset
;
; -----------------
;
GAP
	movwf   DELCNT
	nop
	goto    $+1
	goto    $+1
DELAY2
	goto    $+1
	decfsz  DELCNT, F
	goto    DELAY2
;
	movlw   .26           
	movwf   DELCNT
	nop
	goto    $+1
	goto    $+1
DELAY3
	goto    $+1
	decfsz  DELCNT, F
	goto    DELAY3
;
	movlw   .26           
	movwf   DELCNT
	nop
	goto    $+1
	goto    $+1
DELAY4
	goto    $+1
	decfsz  DELCNT, F
	goto    DELAY4
	return
;
; --------------------
;
SECDEL						; one second delay
	movlw	d'16'
	movwf	S16TMR			; set up delay
SECDL1
	BFNZ	S16TMR,SECDL1
	return
;
; ------------------
;
QSECDL						; quarter second delay
	movlw	d'4'
	movwf	S16TMR			; set up delay
SECDL2
	BFNZ	S16TMR,SECDL2
	return
;
; ------------------
;
QSECDB						; quarter second delay
	movlw	d'4'			; with abort if button down
	movwf	S16TMR			; set up delay
SECDL3
	btfsc	BUTPIN			; button down?
	goto	SECDL4			;   yes
	BFNZ	S16TMR,SECDL3
	return
SECDL4
	clrf	S16TMR
	return
;
; ----------------
;
FLASHR						; 
	bsf		LEDPIN			; LED on
	call	QSECDL
	bcf		LEDPIN			; LED off
	call	QSECDL
	return
;
; ---------------------
;
INTRAN						; wait for timer interrupt (<60ms)
	bcf		FLAGS,TMRFLG
INTRN1
	btfss	FLAGS,TMRFLG    ; TMRFLG is set by intr routine
	goto	INTRN1
	return
;
; ----------------------
;
INIVAR
    movlw   RAMHI			; Highest ram adrs used by program
    movwf   FSR
ZAPRAM
    clrf    INDF            ; Zero the next user ram location
    decf    FSR, F
    BFGEL	FSR,RAMLO,ZAPRAM
#ifdef HANDSET
	movlw	S_SHDN			; Set handset state to off
#else
	movlw	S_ONRX			; Set basestation state to receiving
#endif
	movwf	PSTATE
	movlw	L_TIME			; Set up LED pips timer
	movwf	LEDTMR
	return
;
; ----------------------
;
INITIO
    movlw   CONX
    movwf   INTCON
    movlw   CMCONX
    movwf   CMCON
	movlw	T1CONX
	movwf	T1CON
;
    bsf     STATUS,RP0     ; select bank 1 *******************
	movlw   OPTX
	movwf   OPTION_REG
	movlw   TRISX
	movwf   TRISIO
    movlw   IOCBX
    movwf   IOCB
	movlw	PIE1X
	movwf	PIE1
	movlw	VRCONX
	movwf	VRCON
    bcf     STATUS,RP0     ; select bank 0 *******************
;
	clrf    STATUS
	movf    GPIO,W         ; read state of GPIO to track changes
    bsf     INTCON,T0IE    ; enable timer interrupts
	retfie                 ; enable global interrupts
;
;
; ************************** CONFIGURATION WORD *********************
;
	ORG     0x2007             ; GP3 (MCLR) is an i/o pin.
    DATA    b'00000001000100'  ; code prot, INT 4MHz, WDT off, PWRT on, BOD on
	END						   ;  
